#include "network.h"
#include "error.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <cstring>

int accept_client(int sockfd)
{
    int newsockfd;
    struct sockaddr_in cli_addr;
    socklen_t clilen = sizeof(cli_addr);

    newsockfd = accept(sockfd, (struct sockaddr*)&cli_addr, &clilen);
    if (newsockfd < 0)
    {
        error("ERROR accepting client");
    }

    return newsockfd;
}

int wait_for_client(int& sockfd)
{
    int newsockfd;
    struct sockaddr_in serv_addr;

    // open socket and await connection from client
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
    {
        error("ERROR creating socket");
    }

    bzero((char*)&serv_addr, sizeof(serv_addr));

    // we successfully created the socket, configure it for binding
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(64999);  // convert number from host to network byte order

    // this is a bit of developer QoL so we can iterate more quickly
    // TODO: make it possible to disable this debug/release build configuration
    const int enable = 1;
    if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(int)) < 0)
    {
        error("ERROR setsockopt(SO_REUSEADDR) failed");
    }
    if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEPORT, &enable, sizeof(int)) < 0)
    {
        error("ERROR setsockopt(SO_REUSEPORT) failed");
    }

    // bind the socket
    if (bind(sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0)
    {
        error("ERROR binding socket");
    }

    // successfully bound socket, start listening for connections
    listen(sockfd, 5);

    newsockfd = accept_client(sockfd);
    return newsockfd;
}

void send_message(int sock, const char* message)
{
    int written = write(sock, message, strlen(message));
    if (written < 0)
    {
        error("ERROR sending message to the client");
    }
    printf("SEND %s\n", message);
}

void send_image_dimensions(int sock, unsigned int width, unsigned int height)
{
    // https://linux.die.net/man/3/htons
    width = htonl(width);
    height = htonl(height);

    int written = write(sock, &width, sizeof(uint32_t));
    if (written < 0)
    {
        error("ERROR writing width");
    }
    written = write(sock, &height, sizeof(uint32_t));
    if (written < 0)
    {
        error("ERROR writing height");
    }
}

